@@ -32,7 +32,6 @@
 #include <map>
 #include <typeinfo>
 #include "string.h"
-#include "general.h"
 
 /* === M A C R O S ========================================================= */
 
@@ -65,7 +64,7 @@ class Time;
 class Color;
 struct Segment;
 class BLinePoint;
-class Matrix;
+class Matrix3;
 class BoneWeightPair;
 class WidthPoint;
 class DashItem;
@@ -95,7 +94,7 @@ namespace types_namespace
 	SYNFIG_DECLARE_TYPE_ALIAS(Color)
 	SYNFIG_DECLARE_TYPE_ALIAS(Segment)
 	SYNFIG_DECLARE_TYPE_ALIAS(BLinePoint)
-	SYNFIG_DECLARE_TYPE_ALIAS(Matrix)
+	SYNFIG_DECLARE_TYPE_ALIAS(Matrix3)
 	SYNFIG_DECLARE_TYPE_ALIAS(BoneWeightPair)
 	SYNFIG_DECLARE_TYPE_ALIAS(WidthPoint)
 	SYNFIG_DECLARE_TYPE_ALIAS(DashItem)
@@ -132,6 +131,7 @@ class Operation
 {
 public:
 	typedef void* InternalPointer;
+	typedef const void* ConstInternalPointer;
 
 	enum OperationType {
 		TYPE_NONE,
@@ -141,24 +141,26 @@ public:
 		TYPE_PUT,
 		TYPE_GET,
 		TYPE_COPY,
-		TYPE_COMPARE,
+		TYPE_EQUAL,
+		TYPE_LESS,
 		TYPE_TO_STRING,
 	};
 
 	typedef InternalPointer	(*CreateFunc)	();
-	typedef void			(*DestroyFunc)	(const InternalPointer);
-	typedef void			(*CopyFunc)		(const InternalPointer dest, const InternalPointer src);
-	typedef bool			(*CompareFunc)	(const InternalPointer, const InternalPointer);
-	typedef InternalPointer	(*BinaryFunc)	(const InternalPointer, const InternalPointer);
-	typedef String			(*ToStringFunc)	(const InternalPointer);
+	typedef void			(*DestroyFunc)	(ConstInternalPointer);
+	typedef void			(*CopyFunc)		(InternalPointer dest, ConstInternalPointer src);
+	typedef bool			(*EqualFunc)	(ConstInternalPointer, ConstInternalPointer);
+	typedef bool			(*LessFunc)		(ConstInternalPointer, ConstInternalPointer);
+	typedef InternalPointer	(*BinaryFunc)	(ConstInternalPointer, ConstInternalPointer);
+	typedef String			(*ToStringFunc)	(ConstInternalPointer);
 
 	template<typename T>
 	class GenericFuncs
 	{
 	public:
 		typedef void 		(*SetFunc)		(InternalPointer dest, const T &src);
-		typedef void 		(*PutFunc)		(T &dest, const InternalPointer src);
-		typedef const T&	(*GetFunc)		(const InternalPointer);
+		typedef void 		(*PutFunc)		(T &dest, ConstInternalPointer src);
+		typedef const T&	(*GetFunc)		(ConstInternalPointer);
 	private:
 		GenericFuncs() { }
 	};
@@ -170,25 +172,28 @@ public:
 		static InternalPointer create()
 			{ return new Inner(); }
 		template<typename Inner>
-		static void destroy(const InternalPointer x)
+		static void destroy(ConstInternalPointer x)
 			{ return delete (Inner*)x; }
 		template<typename Inner, typename Outer>
 		static void set(InternalPointer dest, const Outer &src)
 			{ *(Inner*)dest = src; }
 		template<typename Inner, typename Outer>
-		static void put(Outer &dest, const InternalPointer src)
+		static void put(Outer &dest, ConstInternalPointer src)
 			{ dest = static_cast<const Outer&>(*(Inner*)src); }
 		template<typename Inner, typename Outer>
-		static const Outer& get(const InternalPointer x)
+		static const Outer& get(ConstInternalPointer x)
 			{ return static_cast<const Outer&>(*(Inner*)x); }
 		template<typename Inner>
-		static void copy(InternalPointer dest, const InternalPointer src)
+		static void copy(InternalPointer dest, ConstInternalPointer src)
 			{ *(Inner*)dest = *(Inner*)src; }
 		template<typename Inner>
-		static bool compare(InternalPointer a, const InternalPointer b)
+		static bool equal(ConstInternalPointer a, ConstInternalPointer b)
 			{ return *(Inner*)a == *(Inner*)b; }
+		template<typename Inner>
+		static bool less(ConstInternalPointer a, ConstInternalPointer b)
+			{ return *(Inner*)a < *(Inner*)b; }
 		template<typename Inner, String (*Func)(const Inner&)>
-		static String to_string(const InternalPointer x)
+		static String to_string(ConstInternalPointer x)
 			{ return Func(*(const Inner*)x); }
 	private:
 		DefaultFuncs() { }
@@ -232,11 +237,15 @@ public:
 		inline static Description get_copy(TypeId type_a, TypeId type_b)
 			{ return Description(TYPE_COPY, 0, type_a, type_b); }
 		inline static Description get_copy(TypeId type)
-			{ return get_compare(type, type); }
-		inline static Description get_compare(TypeId type_a, TypeId type_b)
-			{ return Description(TYPE_COPY, 0, type_a, type_b); }
-		inline static Description get_compare(TypeId type)
-			{ return get_compare(type, type); }
+			{ return get_copy(type, type); }
+		inline static Description get_equal(TypeId type_a, TypeId type_b)
+			{ return Description(TYPE_EQUAL, 0, type_a, type_b); }
+		inline static Description get_equal(TypeId type)
+			{ return get_equal(type, type); }
+		inline static Description get_less(TypeId type_a, TypeId type_b)
+			{ return Description(TYPE_LESS, 0, type_a, type_b); }
+		inline static Description get_less(TypeId type)
+			{ return get_less(type, type); }
 		inline static Description get_to_string(TypeId type)
 			{ return Description(TYPE_TO_STRING, 0, type); }
 		inline static Description get_binary(OperationType operation_type, TypeId return_type, TypeId type_a, TypeId type_b)
@@ -256,6 +265,7 @@ class Type
 public:
 	enum { NIL = 0 };
 	typedef Operation::InternalPointer InternalPointer;
+	typedef Operation::ConstInternalPointer ConstInternalPointer;
 
 	struct Description
 	{
@@ -400,6 +410,8 @@ private:
 	}
 
 protected:
+	static String local_n(const char *x);
+
 	virtual void initialize_vfunc(Description &description)
 	{
 		description.version = "0.0";
@@ -488,10 +500,14 @@ protected:
 		{ register_operation(Operation::Description::get_copy(type_a, type_b), func); }
 	inline void register_copy(TypeId type, Operation::CopyFunc func)
 		{ register_operation(Operation::Description::get_copy(type), func); }
-	inline void register_compare(TypeId type_a, TypeId type_b, Operation::CompareFunc func)
-		{ register_operation(Operation::Description::get_compare(type_a, type_b), func); }
-	inline void register_compare(TypeId type, Operation::CompareFunc func)
-		{ register_operation(Operation::Description::get_compare(type), func); }
+	inline void register_equal(TypeId type_a, TypeId type_b, Operation::EqualFunc func)
+		{ register_operation(Operation::Description::get_equal(type_a, type_b), func); }
+	inline void register_equal(TypeId type, Operation::EqualFunc func)
+		{ register_operation(Operation::Description::get_equal(type), func); }
+	inline void register_less(TypeId type_a, TypeId type_b, Operation::LessFunc func)
+		{ register_operation(Operation::Description::get_less(type_a, type_b), func); }
+	inline void register_less(TypeId type, Operation::EqualFunc func)
+		{ register_operation(Operation::Description::get_less(type), func); }
 	inline void register_to_string(TypeId type, Operation::ToStringFunc func)
 		{ register_operation(Operation::Description::get_to_string(type), func); }
 	inline void register_binary(Operation::OperationType operation_type, TypeId type_return, TypeId type_a, TypeId type_b, Operation::BinaryFunc func)
@@ -505,72 +521,46 @@ protected:
 		{ register_destroy(identifier, func); }
 	template<typename T>
 	inline void register_set(typename Operation::GenericFuncs<T>::SetFunc func)
-		{ register_set(identifier, func); }
+		{ register_set<T>(identifier, func); }
 	template<typename T>
 	inline void register_put(typename Operation::GenericFuncs<T>::PutFunc func)
-		{ register_put(identifier, func); }
+		{ register_put<T>(identifier, func); }
 	template<typename T>
 	inline void register_get(typename Operation::GenericFuncs<T>::GetFunc func)
-		{ register_get(identifier, func); }
+		{ register_get<T>(identifier, func); }
 	inline void register_copy(Operation::CopyFunc func)
 		{ register_copy(identifier, func); }
-	inline void register_compare(Operation::CompareFunc func)
-		{ register_compare(identifier, func); }
+	inline void register_equal(Operation::EqualFunc func)
+		{ register_equal(identifier, func); }
+	inline void register_less(Operation::EqualFunc func)
+		{ register_less(identifier, func); }
 	inline void register_to_string(Operation::ToStringFunc func)
 		{ register_to_string(identifier, func); }
 
-	// default register
-	inline void register_default(Operation::CreateFunc func)
-		{ register_create(identifier, func); }
-	inline void register_default(Operation::DestroyFunc func)
-		{ register_destroy(identifier, func); }
-	template<typename T>
-	inline void register_default(typename Operation::GenericFuncs<T>::SetFunc func)
-		{ register_set<T>(identifier, func); }
-	template<typename T>
-	inline void register_default(typename Operation::GenericFuncs<T>::PutFunc func)
-		{ register_put<T>(identifier, func); }
-	template<typename T>
-	inline void register_default(typename Operation::GenericFuncs<T>::GetFunc func)
-		{ register_get<T>(identifier, func); }
-	inline void register_default(Operation::CopyFunc func)
-		{ register_copy(identifier, func); }
-	inline void register_default(Operation::CompareFunc func)
-		{ register_compare(identifier, func); }
-	inline void register_default(Operation::ToStringFunc func)
-		{ register_to_string(identifier, func); }
-
-	template<typename Inner, typename Outer, String (*Func)(const Inner&)>
-	inline void register_all()
+	template<typename Inner, typename Outer>
+	inline void register_alias()
 	{
-		register_default(Operation::DefaultFuncs::create<Inner>);
-		register_default(Operation::DefaultFuncs::destroy<Inner>);
-		register_default<Outer>(Operation::DefaultFuncs::set<Inner, Outer>);
-		register_default<Outer>(Operation::DefaultFuncs::put<Inner, Outer>);
-		register_default<Outer>(Operation::DefaultFuncs::get<Inner, Outer>);
-		register_default(Operation::DefaultFuncs::copy<Inner>);
-		register_default(Operation::DefaultFuncs::compare<Inner>);
-		register_default(Operation::DefaultFuncs::to_string<Inner, Func>);
+		register_set<Outer> ( Operation::DefaultFuncs::set<Inner, Outer>      );
+		register_put<Outer> ( Operation::DefaultFuncs::put<Inner, Outer>      );
+		register_get<Outer> ( Operation::DefaultFuncs::get<Inner, Outer>      );
 	}
 
 	template<typename Inner, typename Outer, String (*Func)(const Inner&)>
 	inline void register_all_but_compare()
 	{
-		register_default(Operation::DefaultFuncs::create<Inner>);
-		register_default(Operation::DefaultFuncs::destroy<Inner>);
-		register_default<Outer>(Operation::DefaultFuncs::set<Inner, Outer>);
-		register_default<Outer>(Operation::DefaultFuncs::put<Inner, Outer>);
-		register_default<Outer>(Operation::DefaultFuncs::get<Inner, Outer>);
-		register_default(Operation::DefaultFuncs::copy<Inner>);
-		register_default(Operation::DefaultFuncs::to_string<Inner, Func>);
+		register_create     ( Operation::DefaultFuncs::create<Inner>          );
+		register_destroy    ( Operation::DefaultFuncs::destroy<Inner>         );
+		register_copy       ( Operation::DefaultFuncs::copy<Inner>            );
+		register_to_string  ( Operation::DefaultFuncs::to_string<Inner, Func> );
+		register_alias<Inner, Outer>();
 	}
 
-	template<typename Inner, typename Outer>
-	inline void register_alias()
+	template<typename Inner, typename Outer, String (*Func)(const Inner&)>
+	inline void register_all()
 	{
-		register_default<Outer>(Operation::DefaultFuncs::set<Inner, Outer>);
-		register_default<Outer>(Operation::DefaultFuncs::put<Inner, Outer>);
-		register_default<Outer>(Operation::DefaultFuncs::get<Inner, Outer>);
+		register_all_but_compare<Inner, Outer, Func>();
+		register_equal      ( Operation::DefaultFuncs::equal<Inner>           );
+		register_less       ( Operation::DefaultFuncs::less<Inner>            );
 	}
 
 	template<typename Outer, String (*Func)(const Outer&)>
